@@ -1,16 +1,14 @@
-use std::panic::Location;
-use std::fmt::Debug;
 use std::pin::Pin;
 use std::marker::Unpin;
 use std::future::Future;
 use std::task::{Context, Poll};
-use log::trace;
 use futures_core::stream::Stream;
 use futures_util::stream;
 use futures_util::stream::StreamExt;
 use pin_project::pin_project;
 
 use crate::internal::Map2;
+use crate::signal::Broadcaster;
 use crate::signal_vec::{VecDiff, SignalVec};
 
 
@@ -502,13 +500,24 @@ pub trait SignalExt: Signal {
 
     #[inline]
     #[track_caller]
-    fn debug(self) -> SignalDebug<Self> where Self: Sized, Self::Item: Debug {
+    #[cfg(feature = "debug")]
+    fn debug(self) -> SignalDebug<Self> where Self: Sized, Self::Item: std::fmt::Debug {
         SignalDebug {
             signal: self,
-            location: Location::caller(),
+            location: std::panic::Location::caller(),
         }
     }
 
+    /// A convenience method for calling [`Broadcaster::new`].
+    ///
+    /// This allows you to clone / split a `Signal` into multiple `Signal`s.
+    ///
+    /// See the documentation for [`Broadcaster`] for more details.
+    #[inline]
+    fn broadcast(self) -> Broadcaster<Self> where Self: Sized {
+        Broadcaster::new(self)
+    }
+
     /// A convenience for calling `Signal::poll_change` on `Unpin` types.
     #[inline]
     fn poll_change_unpin(&mut self, cx: &mut Context) -> Poll<Option<Self::Item>> where Self: Unpin + Sized {
@@ -571,13 +580,15 @@ pub fn or<A, B>(left: A, right: B) -> impl Signal<Item = bool>
 #[pin_project]
 #[derive(Debug)]
 #[must_use = "Signals do nothing unless polled"]
+#[cfg(feature = "debug")]
 pub struct SignalDebug<A> {
     #[pin]
     signal: A,
-    location: &'static Location<'static>,
+    location: &'static std::panic::Location<'static>,
 }
 
-impl<A> Signal for SignalDebug<A> where A: Signal, A::Item: Debug {
+#[cfg(feature = "debug")]
+impl<A> Signal for SignalDebug<A> where A: Signal, A::Item: std::fmt::Debug {
     type Item = A::Item;
 
     fn poll_change(self: Pin<&mut Self>, cx: &mut Context) -> Poll<Option<Self::Item>> {
@@ -585,7 +596,7 @@ impl<A> Signal for SignalDebug<A> where A: Signal, A::Item: Debug {
 
         let poll = this.signal.poll_change(cx);
 
-        trace!("[{}] {:#?}", this.location, poll);
+        log::trace!("[{}] {:#?}", this.location, poll);
 
         poll
     }
